"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const GroupNet_1 = require("./GroupNet");
const events_1 = require("events");
const debug_1 = require("debug");
let Debug = debug_1.debug('server-libs');
// 这个负责处理主从关系，后续一定会出现公共模块竞争和决策的问题，如果nodejs的负载能力可以，那么就用nodejs负责路由，否则使用nginx
class MasterSlaver extends events_1.EventEmitter {
    constructor(host, port) {
        super();
        this.ready = false;
        this._initInfo = {};
        this._group = new GroupNet_1.GroupMgr(host, port);
        this._group.on("data", this.onData.bind(this));
        this._group.on('close', this.onClose.bind(this));
        this._group.on('ready', this.onReady.bind(this));
        this._group.on('roletype', this.roletype.bind(this));
    }
    get isMaster() {
        return this._group.isMaster;
    }
    get masterId() {
        return this._group.masterRoleId;
    }
    get roleId() {
        return this._group.roleId;
    }
    init(roleId, groupId, score, info) {
        return __awaiter(this, void 0, void 0, function* () {
            this._initInfo = { roleId: roleId, groupId: groupId, score: score, info: info };
            yield this._group.init();
            yield this.joinGroup();
            this._group.on("connection", this.onConnection.bind(this));
            this._group.on("error", e => {
                Debug(e);
            });
            return true;
        });
    }
    joinGroup() {
        let _this = this;
        this._group.joinGroup(this._initInfo.roleId, this._initInfo.groupId, this._initInfo.score, this._initInfo.info);
        return new Promise(function (resolve, reject) {
            function onError() {
                _this._group.off("open", onOpen);
                reject();
            }
            function onOpen() {
                _this._group.off("error", onError);
                resolve();
            }
            _this._group.once("open", onOpen);
            _this._group.once("error", onError);
        });
    }
    close() {
        this.removeAllListeners();
        this._group.close();
    }
    // master推送数据给普通人 比如通知slave 刷新数据等操作
    controlMsg(cmd, info, requestId) {
        if (!this.isMaster) {
            return false;
        }
        // 推送的时候 master 是不会再发给自己一份的，所以这里需要处理一下
        process.nextTick(() => {
            this.emit('control', cmd, info, this._group.roleId, requestId);
        });
        return this._group.pushBroadcast({
            type: 'control',
            cmd: cmd,
            info: info,
            roleId: this._group.roleId,
            extId: requestId
        });
    }
    // 应答给特定的用户
    answerMsg(roleId, cmd, info, requestId) {
        if (!this.isMaster) {
            return false;
        }
        if (roleId == this._group.roleId) {
            process.nextTick(() => {
                this.emit('answer', cmd, info, roleId, requestId);
            });
            return true;
        }
        return this._group.pushMsg(roleId, {
            type: 'answer',
            cmd: cmd,
            info: info,
            roleId: this._group.roleId,
            extId: requestId
        });
    }
    // normal推送数据给指定用户 主要是执行一些控制命令 比如得到同步信息等，暂时也可以依赖数据库能力
    requestMsg(roleId, cmd, info, requestId) {
        if (roleId == this._group.roleId) {
            process.nextTick(() => {
                this.emit('request', cmd, info, this._group.roleId, requestId);
            });
            return true;
        }
        return this._group.pushMsg(roleId, {
            type: 'request',
            cmd: cmd,
            info: info,
            roleId: this._group.roleId,
            extId: requestId
        });
    }
    /**
     * 通知所有人内容 主要是平级间的，不分主次的命令才可以
     * @param cmd
     * @param info
     */
    normalBroadcast(cmd, info, toSelf = true) {
        // 推送的时候 master 是不会再发给自己一份的，所以这里需要处理一下
        if (toSelf) {
            process.nextTick(() => {
                this.emit('control', cmd, info, this._group.roleId);
            });
        }
        return this._group.pushBroadcast({
            type: 'control',
            cmd: cmd,
            info: info,
            roleId: this._group.roleId
        });
    }
    onData(roleId, info) {
        Debug("onData", roleId, info);
        // 如果是别人发送过来的，那么需要上报recive消息
        this.emit(info.type, info.cmd, info.info, roleId, info.extId);
    }
    onConnection() {
        this._group.joinGroup(this._initInfo.roleId, this._initInfo.groupId, this._initInfo.score, this._initInfo.info);
    }
    onClose() {
        this.ready = false;
    }
    onReady() {
        this.ready = true;
    }
    roletype(isMaster, type) {
        this.emit('roletype', isMaster, type);
    }
    on(event, listener) { super.on(event, listener); return this; }
    once(event, listener) { super.once(event, listener); return this; }
    off(event, listener) { super.off(event, listener); return this; }
}
exports.MasterSlaver = MasterSlaver;
